﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.IO;

namespace MPPMove
{
    class Program
    {
        public const string sSource = "MPPMove";
        public const string sLog = "Application";

        private static void LogEvents(EventLogEntryType logType, string message)
        {
            try
            {
                if (!EventLog.SourceExists(sSource))
                    EventLog.CreateEventSource(sSource, sLog);
                EventLog.WriteEntry(sSource, message, logType);
            }
            catch (Exception) { }
        }

        public static void CopyAll(DirectoryInfo sourceDirectory, DirectoryInfo targetDirectory)
        {
            foreach (FileInfo file in sourceDirectory.GetFiles())
                File.Copy(file.FullName, targetDirectory + "\\" + file.Name);

            foreach (DirectoryInfo sourceSubDir in sourceDirectory.GetDirectories())
            {
                DirectoryInfo nextTargetSubDir = targetDirectory.CreateSubdirectory(sourceSubDir.Name);
                CopyAll(sourceSubDir, nextTargetSubDir);
            }
        }

        /// <summary>
        /// This post processor moves export directory into the specified target directory
        /// </summary>
        /// <param name="args"></param>
        
        static void Main(string[] args)
        {
            try
            {
                if (args.Length != 5)
                    LogEvents(EventLogEntryType.Error, "Invalid argumens, args.Length = " + args.Length.ToString() + ". Expected arguments: <export directory>, <job name>, <step name>, <marshal structure>, <target directory>.");
                else
                {
                    string exportDirectory = args[0];
                    string jobName = args[1];
                    string marshalStructureName = args[2];
                    string stepName = args[3];
                    string targetDirectory = args[4];

                    if (string.IsNullOrEmpty(exportDirectory))
                        LogEvents(EventLogEntryType.Error, "No export directory was specified.");
                    else if (!Directory.Exists(exportDirectory))
                        LogEvents(EventLogEntryType.Error, "The export directory \"" + exportDirectory + "\" does not exist.");
                    else if (string.IsNullOrEmpty(targetDirectory))
                        LogEvents(EventLogEntryType.Error, "No target directory was specified.");
                    else if (!Directory.Exists(targetDirectory))
                        LogEvents(EventLogEntryType.Error, "The target directory \"" + targetDirectory + "\" does not exist.");
                    else
                    {
                        if (string.Compare(Path.GetPathRoot(exportDirectory), Path.GetPathRoot(targetDirectory), true) == 0)
                        {
                            Directory.Move(exportDirectory, targetDirectory.Replace("/", "\\").TrimEnd('\\') + "\\" + Path.GetDirectoryName(exportDirectory));
                        }
                        else
                        {
                            CopyAll(new DirectoryInfo(exportDirectory), new DirectoryInfo(targetDirectory));
                            Directory.Delete(exportDirectory, true);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogEvents(EventLogEntryType.Error, ex.Message);
            }
        }
    }
}
