﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.IO;

namespace MPPZip
{
    class Program
    {
        public const string sSource = "MPPZip";
        public const string sLog = "Application";

        public class Commands
        {
            public const string ZipPassword = "-p";
            public const string DeleteExport = "-d";
            public const string TargetDirectory = "-t";
        }

        private static void LogEvents(EventLogEntryType logType, string message)
        {
            try
            {
                if (!EventLog.SourceExists(sSource))
                    EventLog.CreateEventSource(sSource, sLog);
                EventLog.WriteEntry(sSource, message, logType);
            }
            catch (Exception) { }
        }

        private static string GetCommandValue(string[] args, string command)
        {
            if (args.Length >= 5)
            {
                for (int i = 4; i < args.Length; i++)
                {
                    string arg = args[i];
                    if (!string.IsNullOrEmpty(arg))
                    {
                        string[] parts = arg.Split(' ');
                        foreach (string part in parts)
                        {
                            if (string.Compare(part, command, true) == 0)
                            {
                                return arg.Replace(command, "").Trim();
                            }
                        }
                    }
                }
            }
            return null;
        }

        private static bool HasCommand(string[] args, string command)
        {
            if (args.Length >= 5)
            {
                for (int i = 4; i < args.Length; i++)
                {
                    string arg = args[i];
                    if (!string.IsNullOrEmpty(arg))
                    {
                        string[] parts = arg.Split(' ');
                        foreach (string part in parts)
                        {
                            if (string.Compare(part, command, true) == 0)
                            {
                                return true;
                            }
                        }
                    }
                }
            }
            return false;
        }

        private static string GetPassword(string[] args)
        {
            string password = GetCommandValue(args, Commands.ZipPassword);
            return ((!string.IsNullOrEmpty(password)) ? password : null);
        }

        static void Main(string[] args)
        {
            try
            {
                if (args.Length < 4)
                    LogEvents(EventLogEntryType.Error, "Invalid argumens, args.Length = " + args.Length.ToString() + ". args.Length >= 4 expected (<export directory>, <job name>, <step name>, <marshal structure> + optional arguments).");
                else 
                {
                    string exportDirectory = args[0];
                    string jobName = args[1];
                    string marshalStructureName = args[2];
                    string stepName = args[3];
                    string targetDirectory = GetCommandValue(args, Commands.TargetDirectory);

                    if (!Directory.Exists(exportDirectory))
                        LogEvents(EventLogEntryType.Error, "The export directory \"" + exportDirectory + "\" does not exist.");
                    else if (!string.IsNullOrEmpty(targetDirectory) && !Directory.Exists(targetDirectory))
                        LogEvents(EventLogEntryType.Error, "The target directory \"" + targetDirectory + "\" does not exist.");
                    else
                    {
                        DirectoryInfo exportDirectoryInfo = new DirectoryInfo(exportDirectory);
                        string zipFilePath = Path.GetDirectoryName(exportDirectoryInfo.FullName) + "\\" + exportDirectoryInfo.Name + ".zip";
                        string zipFileName = exportDirectoryInfo.Name + ".zip";

                        ICSharpCode.SharpZipLib.Zip.FastZip fastZip = new ICSharpCode.SharpZipLib.Zip.FastZip();
                        fastZip.Password = GetPassword(args);
                        fastZip.CreateZip(zipFilePath, exportDirectory, true, null);

                        if (!string.IsNullOrEmpty(targetDirectory))
                        {
                            string zipTargetPath = targetDirectory.Replace("/", "\\").TrimEnd('\\') + "\\" + zipFileName;
                            if (string.Compare(Path.GetPathRoot(targetDirectory), Path.GetPathRoot(exportDirectory), true) == 0)
                            {
                                if(File.Exists(zipTargetPath))
                                    File.Delete(zipFilePath);
                                File.Move(zipFilePath, zipTargetPath);
                            }
                            else
                            {
                                File.Copy(zipFilePath, zipTargetPath);
                                File.Delete(zipFilePath);
                            }
                        }
                        if (HasCommand(args, Commands.DeleteExport))
                        {
                            Directory.Delete(exportDirectory, true);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                LogEvents(EventLogEntryType.Error, ex.Message);
            }
        }
    }
}
